/********************************** (C) COPYRIGHT *******************************
* File Name          : main.c
* Author             : WCH
* Version            : V1.0.0
* Date               : 2021/06/06
* Description        : Main program body.
*********************************************************************************
* Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
* Attention: This software (modified or not) and binary are used for 
* microcontroller manufactured by Nanjing Qinheng Microelectronics.
*******************************************************************************/

/*
 *@Note
 task1 and task2 alternate printing
*/

#include "debug.h"
#include "FreeRTOS.h"
#include "task.h"
#include "api.h"
#include "eth_driver.h"
#include "lwip/init.h"
#include "lwip/tcpip.h"
#include "semphr.h"

#include "lwip/dhcp.h"
#include "ethernetif.h"

/* Global define */
#define TASK1_TASK_PRIO     5
#define TASK1_STK_SIZE      256
#define TASK2_TASK_PRIO     5
#define TASK2_STK_SIZE      256

/* Global Variable */
TaskHandle_t Task1Task_Handler;
TaskHandle_t Task2Task_Handler;

/* Semaphore to signal Ethernet Link state update */
SemaphoreHandle_t  Netif_LinkSemaphore = NULL;
/* Ethernet link thread Argument */
struct link_str link_arg;
struct netif gnetif;
ip4_addr_t ipaddr;
ip4_addr_t netmask;
ip4_addr_t gw;

void LwIP_Init(void)
{
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_RNG, ENABLE);
    RNG_Cmd(ENABLE);
    printf("enable rng ok \r\n");

    tcpip_init(NULL, NULL);

    ipaddr.addr = 0;
    netmask.addr = 0;
    gw.addr = 0;

    /* add the network interface (IPv4/IPv6) with RTOS */
    netif_add(&gnetif, &ipaddr, &netmask, &gw, NULL, &ethernetif_init, &tcpip_input);
    /* Registers the default network interface */
    netif_set_default(&gnetif);

    if (netif_is_link_up(&gnetif))
    {
      /* When the netif is fully configured this function must be called */
      netif_set_up(&gnetif);
    }
    else
    {
      /* When the netif link is down this function must be called */
      netif_set_down(&gnetif);
    }

    /* Set the link callback function, this function is called on change of link status*/
    netif_set_link_callback(&gnetif, ethernetif_update_config);

    vSemaphoreCreateBinary(Netif_LinkSemaphore);
    if(Netif_LinkSemaphore==NULL){
        printf("vSemaphoreCreateBinary(Netif_LinkSemaphore) fail\n");
    }

    link_arg.netif = &gnetif;
    link_arg.semaphore = Netif_LinkSemaphore;
    /* Create the Ethernet link handler thread */
    xTaskCreate((TaskFunction_t )ethernetif_set_link,
                    (const char*    )"ethernetif_set_link",
                    (uint16_t       )256,
                    (void*          )&link_arg,
                    (UBaseType_t    )5,
                    NULL);
    //httpd_init();

    printf("Initialized LwIP \r\n");
    dhcp_start(&gnetif);

    int err=ERR_ARG;
       /* Start DHCP negotiation for a network interface (IPv4) */

       if(netif_is_up(&gnetif))
       {
           printf("starting dhcp...\n");
           err = dhcp_start(&gnetif);
       }
       if(err == ERR_OK)
       {
           printf("starting dhcp success!\n");
       }
       else{
           printf("starting dhcp fail!\n");
       }
       while(1){
           while(ip_addr_cmp(&(gnetif.ip_addr),&ipaddr)) //ȴdhcpipЧ
           {
               vTaskDelay(500);
           }
           printf("Local IP :%d.%d.%d.%d\n\n",  \
                       ((gnetif.ip_addr.addr)&0x000000ff),       \
                       (((gnetif.ip_addr.addr)&0x0000ff00)>>8),  \
                       (((gnetif.ip_addr.addr)&0x00ff0000)>>16), \
                       ((gnetif.ip_addr.addr)&0xff000000)>>24);
           printf("Local Netmask :%d.%d.%d.%d\n\n",  \
                       ((gnetif.netmask.addr)&0x000000ff),       \
                       (((gnetif.netmask.addr)&0x0000ff00)>>8),  \
                       (((gnetif.netmask.addr)&0x00ff0000)>>16), \
                       ((gnetif.netmask.addr)&0xff000000)>>24);
           printf("Local GW :%d.%d.%d.%d\n\n",  \
                       ((gnetif.gw.addr)&0x000000ff),       \
                       (((gnetif.gw.addr)&0x0000ff00)>>8),  \
                       (((gnetif.gw.addr)&0x00ff0000)>>16), \
                       ((gnetif.gw.addr)&0xff000000)>>24);
           ip_addr_set(&ipaddr,&(gnetif.ip_addr));
           ip_addr_set(&netmask,&(gnetif.netmask));
           ip_addr_set(&gw,&(gnetif.gw));
           vTaskDelay(500);
       }

}

/*********************************************************************
 * @fn      task1_task
 *
 * @brief   task1 program.
 *
 * @param  *pvParameters - Parameters point of task1
 *
 * @return  none
 */
void task1_task(void *pvParameters)
{
    printf("task1 entry\r\n");
    LwIP_Init();
    printf("task1 delete\r\n");
    vTaskDelete(Task1Task_Handler);
}

/*********************************************************************
 * @fn      task2_task
 *
 * @brief   task2 program.
 *
 * @param  *pvParameters - Parameters point of task2
 *
 * @return  none
 */
void task2_task(void *pvParameters)
{
    while(1)
    {
        printf("task2 entry\r\n");

        vTaskDelay(500);

        vTaskDelay(500);
    }
}

/*********************************************************************
 * @fn      main
 *
 * @brief   Main program.
 *
 * @return  none
 */
int main(void)
{
	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_2);
	SystemCoreClockUpdate();
	Delay_Init();
	USART_Printf_Init(115200);
		
	printf("SystemClk:%d\r\n",SystemCoreClock);
	printf("ChipID:%08x\r\n", DBGMCU_GetCHIPID() );
	printf("FreeRTOS Kernel Version:%s\r\n",tskKERNEL_VERSION_NUMBER);

    printf("LwIP Version: %s \r\n", LWIP_VERSION_STRING);

	/* create two task */
//    xTaskCreate((TaskFunction_t )task2_task,
//                        (const char*    )"task2",
//                        (uint16_t       )TASK2_STK_SIZE,
//                        (void*          )NULL,
//                        (UBaseType_t    )TASK2_TASK_PRIO,
//                        (TaskHandle_t*  )&Task2Task_Handler);

    xTaskCreate((TaskFunction_t )task1_task,
                    (const char*    )"task1",
                    (uint16_t       )TASK1_STK_SIZE,
                    (void*          )NULL,
                    (UBaseType_t    )TASK1_TASK_PRIO,
                    (TaskHandle_t*  )&Task1Task_Handler);
    vTaskStartScheduler();

	while(1)
	{
	    printf("shouldn't run at here!!\n");
	}
}
